import mistune
import notion_client

from notionfier.helpers.notion_params import build_notion_page_params
from notionfier.models.base import NotionObject
from notionfier.modules.renderer import NotionfierRender


class NotionPageBuilder:
    def __init__(self, token: str, renderer: NotionfierRender | None = None, plugins: list[str] | None = None) -> None:
        self.token = token
        renderer = renderer or NotionfierRender()
        if plugins is None:
            plugins = [
                "task_lists",
                "table",
                "url",
                "def_list",
                "strikethrough",
                "footnotes",
            ]
        self.parser = mistune.create_markdown(renderer=renderer, plugins=plugins)

    def parse_content(self, content: str) -> list[NotionObject]:
        return self.parser(content)

    async def acreate_page(self, parent_page_id: str, name: str, content: str):
        client = notion_client.AsyncClient(auth=self.token)
        return await client.pages.create(
            **build_notion_page_params(
                parent_page_id, name, self.parse_content(content)
            )
        )

    def create_page(self, parent_page_id: str, name: str, content: str):
        client = notion_client.Client(auth=self.token)
        return client.pages.create(
            **build_notion_page_params(
                parent_page_id, name, self.parse_content(content)
            )
        )



import dataclasses
from typing import Any

from notionfier.models.block import NotionObject


def build_children_params(children: list[NotionObject]) -> list[dict[str, Any]]:
    return [
        dataclasses.asdict(x, dict_factory=lambda x: {k: v for (k, v) in x if v is not None}) for x in children
    ]


def build_notion_page_params(parent_page_id: str, name: str, children: list[NotionObject]) -> dict[str, Any]:
    return {
        "parent": {"page_id": parent_page_id},
        "properties": {"title": {"title": [{"text": {"content": name}}]}},
        "children": build_children_params(children),
    }



from .block import (
    BlockObject,
    BulletedListItem,
    Code,
    Divider,
    Equation,
    Heading1,
    Heading2,
    Heading3,
    Image,
    NumberedListItem,
    Paragraph,
    Quote,
    Table, TableRow, Todo
)
from .common import Annotation, ExternalFile, LinkObject, RichText, Text
from .enums import CodeLanguage, NumberFormat, TextColor

__all__ = [
    "BlockObject",
    "BulletedListItem",
    "Code",
    "Divider",
    "Equation",
    "Heading1",
    "Heading2",
    "Heading3",
    "Image",
    "NumberedListItem",
    "Paragraph",
    "Quote",
    "Annotation",
    "ExternalFile",
    "LinkObject",
    "RichText",
    "Text",
    "CodeLanguage",
    "NumberFormat",
    "TextColor",
    "Table",
    "TableRow",
    "Todo"
]



from typing import Any, List, Optional, Tuple

from mistune.renderers.html import HTMLRenderer

from notionfier.models import (
    Annotation,
    BlockObject,
    BulletedListItem,
    Code,
    CodeLanguage,
    Divider,
    ExternalFile,
    Heading1,
    Heading2,
    Heading3,
    Image,
    LinkObject,
    NumberedListItem,
    Paragraph,
    Quote,
    RichText,
    Table,
    TableRow,
    Text,
    Todo,
)
from notionfier.models.base import NotionObject


def _split_list_of_notion_objects(
        notion_objects: List[NotionObject],
) -> Tuple[List[RichText], List[BlockObject]]:
    rich_texts: List[RichText] = []
    block_objects: List[BlockObject] = []
    for obj in notion_objects:
        if isinstance(obj, RichText):
            rich_texts.append(obj)
        elif isinstance(obj, BlockObject):
            block_objects.append(obj)
    return rich_texts, block_objects


def _process_annotation(text: RichText, key: str, value: Any) -> RichText:
    if text.annotations is None:
        text.annotations = Annotation()
    setattr(text.annotations, key, value)
    return text


class NotionfierRender(HTMLRenderer):
    name = "Notion"

    def render_token(self, token, state):
        func = self._get_method(token['type'])
        attrs = token.get('attrs')

        if 'raw' in token:
            text = token['raw']
        elif 'children' in token:
            text = self.render_tokens(token['children'], state)
        else:
            if attrs:
                return func(**attrs)
            else:
                return func()
        if attrs:
            return func(text, **attrs)
        else:
            return func(text)

    def render_tokens(self, tokens, state):
        return [item for sublist in self.iter_tokens(tokens, state) for item in sublist]

    def text(self, text: str):
        return [Text(text=Text.Content(content=text))]

    def link(
            self,
            link: str,
            children_objects: Optional[List[NotionObject]] = None,
            title=None,
    ):
        children_objects = children_objects or []
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        result = []
        if len(text_objects) > 0:
            for obj in text_objects:
                assert isinstance(obj, Text)  # todo: check this
                result.append(
                    Text(
                        text=Text.Content(content=obj.text.content, link=LinkObject(url=link)),
                        annotations=obj.annotations,
                    )
                )
        else:
            result.append(Text(text=Text.Content(content=link, link=LinkObject(url=link))))
        return result

    def image(self, src, alt="", title: Optional[List[NotionObject]] = None):
        # todo: image caption
        block = Image(
            image=Image.Content(external=ExternalFile(url=src), caption=[Text(text=Text.Content(content=alt))])
        )
        return [block]

    def emphasis(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        return [_process_annotation(x, "italic", True) for x in text_objects]

    def strong(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        return [_process_annotation(x, "bold", True) for x in text_objects]

    def codespan(self, code: str):
        return [Text(text=Text.Content(content=code), annotations=Annotation(code=True))]

    def linebreak(self):
        return [Text(text=Text.Content(content="\n"))]

    def inline_html(self, html):
        return [Text(text=Text.Content(content=html))]

    def paragraph(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        return [Paragraph(paragraph=Paragraph.Content(rich_text=text_objects, children=block_objects or None))]

    def heading(self, children_objects: List[NotionObject], level: int, **attrs):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        max_level = 2
        if level > max_level:
            level = max_level

        if level == 1:
            return [Heading1(heading_1=Heading1.Content(rich_text=text_objects))]
        else:
            return [Heading2(heading_2=Heading2.Content(rich_text=text_objects))]

    def newline(self):
        return []

    def thematic_break(self):
        return [Divider()]

    def block_text(self, children_objects: List[NotionObject]):
        return children_objects

    def block_code(self, code: str, info: Optional[str] = None):
        block = Code(code=Code.Content(rich_text=[Text(text=Text.Content(content=code))]))
        if info is not None:
            language = info.strip().lower()

            # todo: consider aliases, like js-javascript
            if language in CodeLanguage.__members__:
                block.code.language = CodeLanguage(language)

        return [block]

    def block_quote(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(text_objects) == 0

        rich_text = []
        children = []
        if len(block_objects) > 0:
            first_obj = block_objects[0]
            if isinstance(first_obj, Paragraph):
                rich_text = first_obj.paragraph.rich_text
                children = block_objects[1:]
            elif isinstance(first_obj, Heading1):
                rich_text = first_obj.heading_1.rich_text
                children = block_objects[1:]
            elif isinstance(first_obj, Heading2):
                rich_text = first_obj.heading_2.rich_text
                children = block_objects[1:]
            elif isinstance(first_obj, Heading3):
                rich_text = first_obj.heading_3.rich_text
                children = block_objects[1:]
            else:
                children = block_objects
        return [Quote(quote=Quote.Content(rich_text=rich_text, children=children or None))]

    def block_html(self, html: str):
        return [Paragraph(paragraph=Paragraph.Content(rich_text=[Text(text=Text.Content(content=html))]))]

    def block_error(self, html):
        return [Paragraph(paragraph=Paragraph.Content(rich_text=[Text(text=Text.Content(content=html))]))]

    def list(self, children_objects: List[NotionObject], ordered, **attrs):
        if not ordered:
            return children_objects

        result = []

        for child in children_objects:
            assert isinstance(child, BulletedListItem)
            result.append(
                NumberedListItem(
                    numbered_list_item=NumberedListItem.Content(
                        rich_text=child.bulleted_list_item.rich_text,
                        children=child.bulleted_list_item.children,
                        color=child.bulleted_list_item.color,
                    )
                )
            )
        return result

    def list_item(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        if len(text_objects) == 0 and len(block_objects) > 0:
            first_block = block_objects[0]
            if isinstance(first_block, Paragraph):
                text_objects = first_block.paragraph.rich_text
                block_objects = block_objects[1:]
        return [
            BulletedListItem(
                bulleted_list_item=BulletedListItem.Content(rich_text=text_objects, children=block_objects or None)
            )
        ]

    def footnote_ref(self, key, index, dup):
        return [Text(text=Text.Content(content=f"[{index}]"))]

    def footnotes(self, children_objects: List[NotionObject]):
        result: List[NotionObject] = [Divider()]
        return result + children_objects

    def footnote_item(self, children_objects: List[NotionObject], key, index, is_inline_text):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        if len(text_objects) == 0 and len(block_objects) > 0:
            first_block = block_objects[0]
            if isinstance(first_block, Paragraph):
                text_objects = first_block.paragraph.rich_text
                block_objects = block_objects[1:]
        return [
            NumberedListItem(
                numbered_list_item=NumberedListItem.Content(rich_text=text_objects, children=block_objects)
            )
        ]

    def strikethrough(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        return [_process_annotation(x, "strikethrough", True) for x in text_objects]

    def def_list(self, children_objects: List[NotionObject]):
        return children_objects

    def def_list_header(self, children_objects: List[NotionObject]):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0
        for obj in text_objects:
            if obj.annotations is None:
                obj.annotations = Annotation()
            obj.annotations.bold = True
            obj.annotations.italic = True
        return [Paragraph(paragraph=Paragraph.Content(rich_text=text_objects))]

    def def_list_item(self, children_objects: List[NotionObject]):
        # todo: support multi-paragraph def list items
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        if len(text_objects) == 0 and len(block_objects) > 0:
            first_block = block_objects[0]
            if isinstance(first_block, Paragraph):
                text_objects = first_block.paragraph.rich_text
                block_objects = block_objects[1:]
        result: List[BlockObject] = [Paragraph(paragraph=Paragraph.Content(rich_text=text_objects))]
        return result + block_objects

    def table(self, children_objects: List[NotionObject]):
        assert len(children_objects) > 0
        width = 0
        for obj in children_objects:
            assert isinstance(obj, TableRow)
            width = len(obj.table_row.cells)
        return [
            Table(
                table=Table.Content(
                    table_width=width,
                    has_row_header=False,
                    has_column_header=True,
                    children=children_objects,  # type: ignore
                )
            )
        ]

    def table_head(self, children_objects: List[NotionObject]):
        all_cells: List[List[RichText]] = []
        for obj in children_objects:
            assert isinstance(obj, TableRow)
            all_cells.append(obj.table_row.cells[0])
        return [TableRow(table_row=TableRow.Content(cells=all_cells))]

    def table_body(self, children_objects: List[NotionObject]):
        for obj in children_objects:
            assert isinstance(obj, TableRow)
        return children_objects

    def table_row(self, children_objects: List[NotionObject]):
        all_cells: List[List[RichText]] = []
        for obj in children_objects:
            assert isinstance(obj, TableRow)
            all_cells.append(obj.table_row.cells[0])
        return [TableRow(table_row=TableRow.Content(cells=all_cells))]

    def table_cell(self, children_objects: List[NotionObject], align=None, is_head=False):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        assert len(block_objects) == 0

        # A little hack here: we use `TableRow` as a temporary container for a table cell.
        return [TableRow(table_row=TableRow.Content(cells=[text_objects]))]

    def task_list_item(self, children_objects: List[NotionObject], level: int, checked: bool):
        text_objects, block_objects = _split_list_of_notion_objects(children_objects)
        if len(text_objects) == 0 and len(block_objects) > 0:
            first_block = block_objects[0]
            if isinstance(first_block, Paragraph):
                text_objects = first_block.paragraph.rich_text
                block_objects = block_objects[1:]
        return [Todo(to_do=Todo.Content(rich_text=text_objects, children=block_objects or None, checked=checked))]



from enum import StrEnum, auto


class TextColor(StrEnum):
    default = auto()
    gray = auto()
    brown = auto()
    orange = auto()
    yellow = auto()
    green = auto()
    blue = auto()
    purple = auto()
    pink = auto()
    red = auto()
    gray_background = auto()
    brown_background = auto()
    orange_background = auto()
    yellow_background = auto()
    green_background = auto()
    blue_background = auto()
    purple_background = auto()
    pink_background = auto()
    red_background = auto()


class CodeLanguage(StrEnum):
    abap = auto()
    arduino = auto()
    bash = auto()
    basic = auto()
    c = auto()
    clojure = auto()
    coffeescript = auto()
    cpp = "c++"
    csharp = "c#"
    css = auto()
    dart = auto()
    diff = auto()
    docker = auto()
    elixir = auto()
    elm = auto()
    erlang = auto()
    flow = auto()
    fortran = auto()
    fsharp = "f#"
    gherkin = auto()
    glsl = auto()
    go = auto()
    graphql = auto()
    groovy = auto()
    haskell = auto()
    html = auto()
    java = auto()
    javascript = auto()
    json = auto()
    julia = auto()
    kotlin = auto()
    latex = auto()
    less = auto()
    lisp = auto()
    livescript = auto()
    lua = auto()
    makefile = auto()
    markdown = auto()
    markup = auto()
    matlab = auto()
    mermaid = auto()
    nix = auto()
    objective_c = "objective-c"
    ocaml = auto()
    pascal = auto()
    perl = auto()
    php = auto()
    plain_text = "plain text"
    powershell = auto()
    prolog = auto()
    protobuf = auto()
    python = auto()
    r = auto()
    reason = auto()
    ruby = auto()
    rust = auto()
    sass = auto()
    scala = auto()
    scheme = auto()
    scss = auto()
    shell = auto()
    sql = auto()
    swift = auto()
    typescript = auto()
    vb_net = "vb.net"
    verilog = auto()
    vhdl = auto()
    visual_basic = "visual basic"
    webassembly = auto()
    xml = auto()
    yaml = auto()


class NumberFormat(StrEnum):
    number = auto()
    number_with_commas = auto()
    percent = auto()
    dollar = auto()
    canadian_dollar = auto()
    euro = auto()
    pound = auto()
    yen = auto()
    ruble = auto()
    rupee = auto()
    won = auto()
    yuan = auto()
    real = auto()
    lira = auto()
    rupiah = auto()
    franc = auto()
    hong_kong_dollar = auto()
    new_zealand_dollar = auto()
    krona = auto()
    norwegian_krone = auto()
    mexican_peso = auto()
    rand = auto()
    new_taiwan_dollar = auto()
    danish_krone = auto()
    zloty = auto()
    baht = auto()
    forint = auto()
    koruna = auto()
    shekel = auto()
    chilean_peso = auto()
    philippine_peso = auto()
    dirham = auto()
    colombian_peso = auto()
    riyal = auto()
    ringgit = auto()
    leu = auto()
    argentine_peso = auto()
    uruguayan_peso = auto()


class SelectOptionColor(StrEnum):
    default = auto()
    gray = auto()
    brown = auto()
    orange = auto()
    yellow = auto()
    green = auto()
    blue = auto()
    purple = auto()
    pink = auto()
    red = auto()



import os

import click

from notionfier import NotionPageBuilder


@click.group()
def cli():
    pass


@cli.command("import")
@click.option("-t", "--token", type=str, required=True, help="The Notion auth token.")
@click.option("-pid", "--parent-page-id", type=str, required=True, help="The page id of the parent page.")
@click.option("-f", "--file-path", type=str, required=True, help="Path to your markdown file.")
def import_notion(token: str, parent_page_id: str, file_path: str):
    with open(file_path, encoding="utf-8") as fi:
        content = fi.read()

    builder = NotionPageBuilder(token)
    builder.create_page(parent_page_id, os.path.basename(file_path), content)


def main():
    cli()


if __name__ == "__main__":
    main()



import dataclasses
from typing import Any, Dict, List, Optional

from .base import NotionObject
from .common import ExternalFile, RichText
from .enums import CodeLanguage, TextColor


@dataclasses.dataclass
class BlockObject(NotionObject):
    pass


@dataclasses.dataclass
class Paragraph(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        children: Optional[List["BlockObject"]] = None
        color: TextColor = TextColor.default

    paragraph: Content


@dataclasses.dataclass
class Heading1(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        color: TextColor = TextColor.default

    heading_1: Content


@dataclasses.dataclass
class Heading2(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        color: TextColor = TextColor.default

    heading_2: Content


@dataclasses.dataclass
class Heading3(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        color: TextColor = TextColor.default

    heading_3: Content


# todo: Callout blocks


@dataclasses.dataclass
class Quote(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        color: TextColor = TextColor.default
        children: Optional[List["BlockObject"]] = None

    quote: Content


@dataclasses.dataclass
class BulletedListItem(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        children: Optional[List["BlockObject"]] = None
        color: TextColor = TextColor.default

    bulleted_list_item: Content


@dataclasses.dataclass
class NumberedListItem(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        children: Optional[List["BlockObject"]] = None
        color: TextColor = TextColor.default

    numbered_list_item: Content


@dataclasses.dataclass
class Todo(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        checked: bool
        children: Optional[List["BlockObject"]] = None
        color: TextColor = TextColor.default

    to_do: Content


@dataclasses.dataclass
class Code(BlockObject):
    @dataclasses.dataclass
    class Content:
        rich_text: List[RichText]
        caption: Optional[List[RichText]] = None
        language: CodeLanguage = CodeLanguage.plain_text

    code: Content


@dataclasses.dataclass
class Image(BlockObject):
    @dataclasses.dataclass
    class Content:
        external: ExternalFile
        caption: List[RichText] = dataclasses.field(default_factory=lambda: [])

    image: Content


@dataclasses.dataclass
class Equation(BlockObject):
    @dataclasses.dataclass
    class Content:
        expression: str

    equation: Content


@dataclasses.dataclass
class Divider(BlockObject):
    divider: Dict[str, Any] = dataclasses.field(default_factory=lambda: {})


@dataclasses.dataclass
class TableRow(NotionObject):
    @dataclasses.dataclass
    class Content:
        cells: List[List[RichText]]

    table_row: Content


@dataclasses.dataclass
class Table(BlockObject):
    @dataclasses.dataclass
    class Content:
        table_width: int
        has_column_header: bool
        has_row_header: bool
        children: List[TableRow]

    table: Content



import dataclasses
from typing import Optional

from .base import NotionObject
from .enums import TextColor


# Rich text objects
@dataclasses.dataclass
class Annotation(NotionObject):
    bold: Optional[bool] = False
    italic: Optional[bool] = False
    strikethrough: Optional[bool] = False
    underline: Optional[bool] = False
    code: Optional[bool] = False
    color: Optional[TextColor] = None


@dataclasses.dataclass
class RichText(NotionObject):
    annotations: Optional[Annotation] = None


@dataclasses.dataclass
class LinkObject(NotionObject):
    url: str


@dataclasses.dataclass
class Text(RichText):
    @dataclasses.dataclass
    class Content:
        content: str = ""
        link: Optional[LinkObject] = None

    text: Content = dataclasses.field(default_factory=Content)


@dataclasses.dataclass
class ExternalFile(NotionObject):
    url: str


# todo: Mention object
# todo: Equation object
# todo: Link preview mention object
